import { sprites, animation, cellSize, previewBorder } from '@/application/state';
import { getSpriteOffset } from '@/application/utilities';

/**
 * Start the animation
 */
export function startAnimation() {
  if (sprites.value.length === 0 || !animation.canvas) return;

  animation.isPlaying = true;
  animation.lastFrameTime = performance.now();
  animation.manualUpdate = false;

  animation.canvas.width = cellSize.width;
  animation.canvas.height = cellSize.height;

  // Start the animation loop without resetting sprite offset
  animationLoop();
}

/**
 * Stop the animation
 */
export function stopAnimation() {
  animation.isPlaying = false;

  if (animation.animationId) {
    cancelAnimationFrame(animation.animationId);
    animation.animationId = null;
  }
}

/**
 * Render a specific frame of the animation
 */
export function renderAnimationFrame(frameIndex: number, showAllSprites = false, spriteOffset = { x: 0, y: 0 }) {
  if (sprites.value.length === 0 || !animation.canvas || !animation.ctx) return;

  // Resize the animation canvas to match the cell size if needed
  if (animation.canvas.width !== cellSize.width || animation.canvas.height !== cellSize.height) {
    animation.canvas.width = cellSize.width;
    animation.canvas.height = cellSize.height;
  }

  // Clear the canvas
  animation.ctx.clearRect(0, 0, animation.canvas.width, animation.canvas.height);

  // Draw background (transparent by default)
  animation.ctx.fillStyle = 'transparent';
  animation.ctx.fillRect(0, 0, animation.canvas.width, animation.canvas.height);

  // Keep pixel art sharp
  animation.ctx.imageSmoothingEnabled = false;

  // Draw background sprites with reduced opacity if requested
  if (showAllSprites && sprites.value.length > 1) {
    renderBackgroundSprites(frameIndex);
  }

  // Get the current sprite and render it
  renderCurrentSprite(frameIndex, spriteOffset);

  // Draw border around the cell if enabled
  renderPreviewBorder();
}

/**
 * Draw all sprites except the current one with reduced opacity
 */
function renderBackgroundSprites(frameIndex: number) {
  if (!animation.ctx) return;

  // Save the current context state
  animation.ctx.save();

  // Set global alpha for background sprites
  animation.ctx.globalAlpha = 0.3;

  // Draw all sprites except the current one
  sprites.value.forEach((sprite, index) => {
    if (index !== frameIndex) {
      const spriteCellX = Math.floor(sprite.x / cellSize.width);
      const spriteCellY = Math.floor(sprite.y / cellSize.height);

      // Calculate precise offset for pixel-perfect rendering
      const spriteOffsetX = Math.round(sprite.x - spriteCellX * cellSize.width);
      const spriteOffsetY = Math.round(sprite.y - spriteCellY * cellSize.height);

      // Draw the sprite with transparency
      animation.ctx?.drawImage(sprite.img, spriteOffsetX, spriteOffsetY);
    }
  });

  // Restore the context to full opacity
  animation.ctx.restore();
}

/**
 * Render the current sprite at full opacity
 */
function renderCurrentSprite(frameIndex: number, spriteOffset: { x: number; y: number }) {
  if (!animation.ctx) return;

  // Get the current sprite
  const currentSprite = sprites.value[frameIndex % sprites.value.length];

  // Draw the current sprite at full opacity at the specified position
  animation.ctx.drawImage(currentSprite.img, spriteOffset.x, spriteOffset.y);
}

/**
 * Render a border around the animation preview if enabled
 */
function renderPreviewBorder() {
  if (!animation.ctx || !animation.canvas || !previewBorder.enabled) return;

  animation.ctx.strokeStyle = previewBorder.color;
  animation.ctx.lineWidth = previewBorder.width;

  // Use pixel-perfect coordinates for the border (0.5 offset for crisp lines)
  const x = 0.5;
  const y = 0.5;
  const width = animation.canvas.width - 1;
  const height = animation.canvas.height - 1;

  animation.ctx.strokeRect(x, y, width, height);
}

/**
 * Animation loop for continuous playback
 */
export function animationLoop(timestamp?: number) {
  if (!animation.isPlaying) return;

  const currentTime = timestamp || performance.now();
  const elapsed = currentTime - animation.lastFrameTime;
  const frameInterval = 1000 / animation.frameRate;

  if (elapsed >= frameInterval) {
    animation.lastFrameTime = currentTime;

    if (sprites.value.length > 0) {
      // Get the stored offset for the current frame
      const frameOffset = getSpriteOffset(animation.currentFrame);

      // Render the current frame with its offset
      renderAnimationFrame(animation.currentFrame, false, frameOffset);

      // Move to the next frame
      animation.currentFrame = (animation.currentFrame + 1) % sprites.value.length;

      // Update the slider position if available
      if (animation.slider) {
        animation.slider.value = animation.currentFrame.toString();
      }
    }
  }

  animation.animationId = requestAnimationFrame(animationLoop);
}
