// canvasOperations.ts
import { sprites, canvas, ctx, cellSize, columns, zoomLevel, previewBorder } from '@/application/state';
import { logger, getSpriteOffset, isImageReady, getPixelPerfectCoordinate, showNotification } from '@/application/utilities';

/**
 * Update the canvas size based on sprites and cell size
 */
export function updateCanvasSize() {
  if (!canvas.value) {
    logger.warn('Canvas not available for size update');
    return;
  }

  if (sprites.value.length === 0) {
    return;
  }

  try {
    const totalSprites = sprites.value.length;
    const cols = columns.value;
    const rows = Math.ceil(totalSprites / cols);

    if (cellSize.width <= 0 || cellSize.height <= 0) {
      logger.error('Invalid cell size for canvas update', cellSize);
      return;
    }

    const newWidth = cols * cellSize.width;
    const newHeight = rows * cellSize.height;

    // Ensure the canvas is large enough to display all sprites
    if (canvas.value.width !== newWidth || canvas.value.height !== newHeight) {
      canvas.value.width = newWidth;
      canvas.value.height = newHeight;

      // Emit an event to update the wrapper dimensions
      window.dispatchEvent(
        new CustomEvent('canvas-size-updated', {
          detail: { width: newWidth, height: newHeight },
        })
      );
    }
  } catch (error) {
    logger.error('Error updating canvas size:', error);
  }
}

/**
 * Draw the grid on the canvas
 */
export function drawGrid() {
  if (!ctx.value || !canvas.value) return;

  ctx.value.strokeStyle = '#333';
  ctx.value.lineWidth = 1 / zoomLevel.value; // Adjust line width based on zoom level

  // Calculate the visible area based on zoom level
  const visibleWidth = canvas.value.width / zoomLevel.value;
  const visibleHeight = canvas.value.height / zoomLevel.value;

  // Draw vertical lines - ensure pixel-perfect grid lines
  for (let x = 0; x <= visibleWidth; x += cellSize.width) {
    const pixelX = getPixelPerfectCoordinate(x);
    ctx.value.beginPath();
    ctx.value.moveTo(pixelX, 0);
    ctx.value.lineTo(pixelX, visibleHeight);
    ctx.value.stroke();
  }

  // Draw horizontal lines - ensure pixel-perfect grid lines
  for (let y = 0; y <= visibleHeight; y += cellSize.height) {
    const pixelY = getPixelPerfectCoordinate(y);
    ctx.value.beginPath();
    ctx.value.moveTo(0, pixelY);
    ctx.value.lineTo(visibleWidth, pixelY);
    ctx.value.stroke();
  }
}

/**
 * Render the spritesheet preview on the canvas
 */
export function renderSpritesheetPreview(showGrid = true) {
  if (!ctx.value || !canvas.value) {
    logger.error('Canvas or context not available for rendering, will retry when ready');
    setTimeout(() => {
      if (ctx.value && canvas.value) {
        renderSpritesheetPreview(showGrid);
      }
    }, 100);
    return;
  }

  if (sprites.value.length === 0) return;

  try {
    // Make sure the canvas size is correct before rendering
    updateCanvasSize();

    // Clear the canvas
    ctx.value.clearRect(0, 0, canvas.value.width, canvas.value.height);

    if (showGrid) {
      drawGrid();
    }

    // First, collect all occupied cells
    const occupiedCells = new Set<string>();
    sprites.value.forEach(sprite => {
      const cellX = Math.floor(sprite.x / cellSize.width);
      const cellY = Math.floor(sprite.y / cellSize.height);
      occupiedCells.add(`${cellX},${cellY}`);
    });

    // Draw each sprite - remove the zoom scaling from context
    sprites.value.forEach((sprite, index) => {
      try {
        if (!sprite.img) {
          logger.warn(`Sprite at index ${index} has no image, skipping render`);
          return;
        }

        // Check if sprite is within canvas bounds
        if (sprite.x >= 0 && sprite.y >= 0 && sprite.x + sprite.width <= canvas.value!.width && sprite.y + sprite.height <= canvas.value!.height) {
          renderSpriteOnCanvas(sprite, index);
        } else {
          logger.warn(`Sprite at index ${index} is outside canvas bounds: sprite(${sprite.x},${sprite.y}) canvas(${canvas.value!.width},${canvas.value!.height})`);
        }
      } catch (spriteError) {
        logger.error(`Error rendering sprite at index ${index}:`, spriteError);
      }
    });

    // Draw borders around occupied cells if enabled (preview only)
    drawPreviewBorders(occupiedCells);
  } catch (error) {
    logger.error('Error in renderSpritesheetPreview:', error);
  }
}

/**
 * Draw a single sprite on the canvas
 */
function renderSpriteOnCanvas(sprite: any, index: number) {
  if (!ctx.value || !canvas.value) return;

  if (isImageReady(sprite.img)) {
    // For pixel art, ensure we're drawing at exact pixel boundaries
    const x = Math.round(sprite.x);
    const y = Math.round(sprite.y);

    // Get the frame-specific offset for this sprite
    const frameOffset = getSpriteOffset(index);

    // Calculate the maximum allowed offset based on sprite and cell size
    const maxOffsetX = Math.max(0, cellSize.width - sprite.width);
    const maxOffsetY = Math.max(0, cellSize.height - sprite.height);

    // Constrain the offset to prevent out-of-bounds positioning
    const constrainedOffsetX = Math.max(0, Math.min(maxOffsetX, frameOffset.x));
    const constrainedOffsetY = Math.max(0, Math.min(maxOffsetY, frameOffset.y));

    // Update the frame offset with the constrained values
    frameOffset.x = constrainedOffsetX;
    frameOffset.y = constrainedOffsetY;

    // Apply the constrained offset to the sprite position
    const finalX = x + constrainedOffsetX;
    const finalY = y + constrainedOffsetY;

    // Draw the image at its final position with pixel-perfect rendering
    ctx.value.imageSmoothingEnabled = false; // Keep pixel art sharp
    ctx.value.drawImage(sprite.img, finalX, finalY, sprite.width, sprite.height);
  } else {
    logger.warn(`Sprite image ${index} not fully loaded, setting onload handler`);
    sprite.img.onload = () => {
      if (ctx.value && canvas.value) {
        renderSpriteOnCanvas(sprite, index);
      }
    };
  }
}

/**
 * Draw preview borders around occupied cells if enabled
 */
function drawPreviewBorders(occupiedCells: Set<string>) {
  if (!ctx.value || !canvas.value || !previewBorder.enabled || occupiedCells.size === 0) return;

  ctx.value.strokeStyle = previewBorder.color;
  ctx.value.lineWidth = previewBorder.width / zoomLevel.value; // Adjust for zoom

  // Draw borders around each occupied cell
  occupiedCells.forEach(cellKey => {
    const [cellX, cellY] = cellKey.split(',').map(Number);

    // Calculate pixel-perfect coordinates for the cell
    // Add 0.5 to align with pixel boundaries for crisp lines
    const x = getPixelPerfectCoordinate(cellX * cellSize.width);
    const y = getPixelPerfectCoordinate(cellY * cellSize.height);

    // Adjust width and height to ensure the border is inside the cell
    const width = cellSize.width - 1;
    const height = cellSize.height - 1;

    ctx.value!.strokeRect(x, y, width, height);
  });
}

/**
 * Download the spritesheet as a PNG
 */
export function downloadSpritesheet() {
  if (sprites.value.length === 0 || !canvas.value) {
    showNotification('No sprites to download', 'error');
    return;
  }

  const tempCanvas = document.createElement('canvas');
  tempCanvas.width = canvas.value.width;
  tempCanvas.height = canvas.value.height;
  const tempCtx = tempCanvas.getContext('2d');

  if (!tempCtx) {
    showNotification('Failed to create download context', 'error');
    return;
  }

  tempCtx.clearRect(0, 0, tempCanvas.width, tempCanvas.height);

  // Ensure pixel art remains sharp in the downloaded file
  tempCtx.imageSmoothingEnabled = false;

  sprites.value.forEach((sprite, index) => {
    // Get the frame-specific offset for this sprite
    const frameOffset = getSpriteOffset(index);

    // Calculate the cell coordinates for this sprite
    const cellX = Math.floor(sprite.x / cellSize.width);
    const cellY = Math.floor(sprite.y / cellSize.height);

    // Calculate the base position within the cell
    const baseX = cellX * cellSize.width;
    const baseY = cellY * cellSize.height;

    // Calculate the maximum allowed offset based on sprite and cell size
    // This prevents sprites from going out of bounds
    const maxOffsetX = Math.max(0, cellSize.width - sprite.width);
    const maxOffsetY = Math.max(0, cellSize.height - sprite.height);

    // Constrain the offset to prevent out-of-bounds positioning
    const constrainedOffsetX = Math.max(0, Math.min(maxOffsetX, frameOffset.x));
    const constrainedOffsetY = Math.max(0, Math.min(maxOffsetY, frameOffset.y));

    // Apply the constrained offset to the base position
    const finalX = baseX + constrainedOffsetX;
    const finalY = baseY + constrainedOffsetY;

    // Draw the sprite at the calculated position
    tempCtx.drawImage(sprite.img, finalX, finalY, sprite.width, sprite.height);
  });

  const link = document.createElement('a');
  link.download = 'spritesheet.png';
  link.href = tempCanvas.toDataURL('image/png');
  document.body.appendChild(link);
  link.click();
  document.body.removeChild(link);

  showNotification('Spritesheet downloaded successfully');
}

/**
 * Zoom-related operations
 */
export function zoomIn() {
  // Increase zoom level by 0.1, max 3.0 (300%)
  zoomLevel.value = Math.min(3.0, zoomLevel.value + 0.1);
  renderSpritesheetPreview();
  showNotification(`Zoom: ${Math.round(zoomLevel.value * 100)}%`);
}

export function zoomOut() {
  // Decrease zoom level by 0.1, min 0.5 (50%)
  zoomLevel.value = Math.max(0.5, zoomLevel.value - 0.1);
  renderSpritesheetPreview();
  showNotification(`Zoom: ${Math.round(zoomLevel.value * 100)}%`);
}

export function resetZoom() {
  // Reset to default zoom level (100%)
  zoomLevel.value = 1;
  renderSpritesheetPreview();
  showNotification('Zoom reset to 100%');
}
