import { spriteOffsets, notification } from '@/application/state';
import { type Sprite } from '@/application/types';

/**
 * Logger utility with consistent error format
 */
export const logger = {
  warn: (message: string, details?: any) => {
    console.warn(`Spritesheet: ${message}`, details || '');
  },
  error: (message: string, error?: any) => {
    console.error(`Spritesheet: ${message}`, error || '');
  },
};

/**
 * Safely execute a function with error handling
 */
export function safeExecute<T>(fn: () => T, errorMessage: string): T | undefined {
  try {
    return fn();
  } catch (error) {
    logger.error(errorMessage, error);
    return undefined;
  }
}

/**
 * Show a notification
 */
export function showNotification(message: string, type: 'success' | 'error' = 'success') {
  notification.message = message;
  notification.type = type;
  notification.isVisible = true;

  setTimeout(() => {
    notification.isVisible = false;
  }, 3000);
}

/**
 * Get the offset for a specific frame
 */
export function getSpriteOffset(frameIndex: number) {
  if (!spriteOffsets.has(frameIndex)) {
    spriteOffsets.set(frameIndex, { x: 0, y: 0 });
  }
  return spriteOffsets.get(frameIndex)!;
}

/**
 * Check if image is fully loaded and ready to use
 */
export function isImageReady(img: HTMLImageElement): boolean {
  return img.complete && img.naturalWidth !== 0;
}

/**
 * Get pixel-perfect coordinates aligned to pixel boundaries
 */
export function getPixelPerfectCoordinate(value: number): number {
  return Math.floor(value) + 0.5;
}

/**
 * Create a sprite object from a file
 */
export function createSpriteFromFile(file: File, index: number): Promise<Sprite> {
  return new Promise((resolve, reject) => {
    // Create a URL for the file
    const objectUrl = URL.createObjectURL(file);

    const img = new Image();

    // Set up event handlers
    img.onload = () => {
      // Verify the image has loaded properly
      if (img.width === 0 || img.height === 0) {
        logger.error('Image loaded with invalid dimensions:', { name: file.name, width: img.width, height: img.height });
        URL.revokeObjectURL(objectUrl);
        reject(new Error(`Image has invalid dimensions: ${file.name}`));
        return;
      }

      // Create the sprite object
      const sprite: Sprite = {
        img,
        width: img.width,
        height: img.height,
        x: 0,
        y: 0,
        name: file.name,
        id: `sprite-${Date.now()}-${Math.random().toString(36).substr(2, 9)}`,
        uploadOrder: index,
      };

      // Keep the objectUrl reference and don't revoke it yet
      // The image is still needed for rendering later
      resolve(sprite);
    };

    img.onerror = error => {
      logger.error('Error loading image:', { name: file.name, error });
      URL.revokeObjectURL(objectUrl);
      reject(new Error(`Failed to load image: ${file.name}`));
    };

    // Set the source to the object URL
    img.src = objectUrl;
  });
}

/**
 * Process multiple files and create sprites
 */
export async function processImageFiles(files: FileList): Promise<{ newSprites: Sprite[]; errorCount: number }> {
  const imageFiles = Array.from(files).filter(file => file.type.startsWith('image/'));

  if (imageFiles.length === 0) {
    return { newSprites: [], errorCount: 0 };
  }

  const newSprites: Sprite[] = [];
  let errorCount = 0;

  for (let i = 0; i < imageFiles.length; i++) {
    const file = imageFiles[i];

    try {
      const sprite = await createSpriteFromFile(file, i);
      newSprites.push(sprite);
    } catch (error) {
      errorCount++;
      logger.error('Error loading sprite:', error);
    }
  }

  return { newSprites, errorCount };
}

/**
 * Truncate text to a specific length and add ellipsis if needed
 */
export function truncateText(text: string, maxLength: number = 15): string {
  return text.length > maxLength ? `${text.substring(0, maxLength)}...` : text;
}
