import { ref, computed, watch, type Ref } from 'vue';
import { type Sprite, type AnimationState } from '@/application/types';

export function useAnimation(sprites: Ref<Sprite[]>, animation: Ref<AnimationState>, onUpdateFrame: (frameIndex: number, offset: { x: number; y: number }) => void, getSpriteOffset: (frameIndex: number) => { x: number; y: number }) {
  // State
  const currentFrame = ref(0);
  const showAllSprites = ref(false);

  // Computed
  const currentFrameDisplay = computed(() => {
    if (sprites.value.length === 0) return '0 / 0';
    return `${currentFrame.value + 1} / ${sprites.value.length}`;
  });

  // Methods
  const startAnimation = () => {
    if (sprites.value.length === 0) return;
    animation.value.isPlaying = true;
    animation.value.lastFrameTime = performance.now();
    animation.value.manualUpdate = false;

    if (animation.value.canvas) {
      animation.value.canvas.width = animation.value.canvas.width; // Reset canvas
    }

    // Start animation loop
    if (!animation.value.animationId) {
      animationLoop();
    }
  };

  const stopAnimation = () => {
    animation.value.isPlaying = false;

    if (animation.value.animationId) {
      cancelAnimationFrame(animation.value.animationId);
      animation.value.animationId = null;
    }
  };

  const animationLoop = (timestamp?: number) => {
    if (!animation.value.isPlaying) return;

    const currentTime = timestamp || performance.now();
    const elapsed = currentTime - animation.value.lastFrameTime;
    const frameInterval = 1000 / animation.value.frameRate;

    if (elapsed >= frameInterval) {
      animation.value.lastFrameTime = currentTime;

      if (sprites.value.length > 0) {
        // Get the stored offset for the current frame
        const frameOffset = getSpriteOffset(animation.value.currentFrame);

        // Update frame with offset
        onUpdateFrame(animation.value.currentFrame, frameOffset);

        // Move to the next frame
        animation.value.currentFrame = (animation.value.currentFrame + 1) % sprites.value.length;
        currentFrame.value = animation.value.currentFrame;

        // Update slider if available
        if (animation.value.slider) {
          animation.value.slider.value = animation.value.currentFrame.toString();
        }
      }
    }

    animation.value.animationId = requestAnimationFrame(animationLoop);
  };

  const handleFrameChange = () => {
    if (animation.value.isPlaying) {
      stopAnimation();
    }

    // Ensure frame is within bounds
    currentFrame.value = Math.max(0, Math.min(currentFrame.value, sprites.value.length - 1));
    animation.value.currentFrame = currentFrame.value;
    updateCurrentFrame();
  };

  const handleFrameRateChange = () => {
    // If animation is currently playing, restart it with the new frame rate
    if (animation.value.isPlaying) {
      stopAnimation();
      startAnimation();
    }
  };

  const updateCurrentFrame = () => {
    // Ensure frame is within bounds
    currentFrame.value = Math.max(0, Math.min(currentFrame.value, sprites.value.length - 1));
    animation.value.currentFrame = currentFrame.value;
    animation.value.manualUpdate = true;

    // Get the offset for the current frame
    const offset = getSpriteOffset(currentFrame.value);

    // Update frame with the current offset
    onUpdateFrame(currentFrame.value, offset);
  };

  const nextFrame = () => {
    if (sprites.value.length === 0) return;
    currentFrame.value = (currentFrame.value + 1) % sprites.value.length;
    updateCurrentFrame();
  };

  const prevFrame = () => {
    if (sprites.value.length === 0) return;
    currentFrame.value = (currentFrame.value - 1 + sprites.value.length) % sprites.value.length;
    updateCurrentFrame();
  };

  // Keep currentFrame in sync with animation.currentFrame
  watch(
    () => animation.value.currentFrame,
    newVal => {
      currentFrame.value = newVal;
    }
  );

  return {
    currentFrame,
    showAllSprites,
    currentFrameDisplay,
    startAnimation,
    stopAnimation,
    handleFrameChange,
    handleFrameRateChange,
    updateCurrentFrame,
    nextFrame,
    prevFrame,
  };
}
