import { onMounted, onBeforeUnmount, type Ref } from 'vue';
import { type Sprite, type AnimationState } from '@/application/types';

export function useKeyboardShortcuts(
  isModalOpen: Ref<boolean>,
  sprites: Ref<Sprite[]>,
  animation: Ref<AnimationState>,
  closeModal: () => void,
  startAnimation: () => void,
  stopAnimation: () => void,
  nextFrame: () => void,
  prevFrame: () => void,
  zoomIn: () => void,
  zoomOut: () => void,
  resetZoom: () => void,
  resetSpritePosition: () => void,
  panViewport: (direction: 'left' | 'right' | 'up' | 'down', amount?: number) => void,
  showNotification: (message: string, type?: 'success' | 'error') => void
) {
  const handleKeyDown = (e: KeyboardEvent) => {
    if (!isModalOpen.value) return;

    // Modal control
    if (e.key === 'Escape') {
      closeModal();
      return;
    }

    // Animation control
    if (e.key === ' ' || e.key === 'Spacebar') {
      // Toggle play/pause
      if (animation.value.isPlaying) {
        stopAnimation();
      } else if (sprites.value.length > 0) {
        startAnimation();
      }
      e.preventDefault();
      return;
    }

    // Frame navigation
    if (e.key === 'ArrowRight' && !animation.value.isPlaying && sprites.value.length > 0) {
      nextFrame();
      e.preventDefault();
      return;
    }

    if (e.key === 'ArrowLeft' && !animation.value.isPlaying && sprites.value.length > 0) {
      prevFrame();
      e.preventDefault();
      return;
    }

    // Zoom controls
    if (e.key === '+' || e.key === '=') {
      zoomIn();
      e.preventDefault();
      return;
    }

    if (e.key === '-' || e.key === '_') {
      zoomOut();
      e.preventDefault();
      return;
    }

    if (e.key === '0') {
      resetZoom();
      e.preventDefault();
      return;
    }

    // Reset sprite position
    if (e.key === 'r') {
      if (e.shiftKey) {
        // Shift+R: Reset sprite position only
        resetSpritePosition();
      } else {
        // R: Reset both sprite position and viewport
        resetSpritePosition();
        resetZoom();
        showNotification('View and position reset');
      }
      e.preventDefault();
      return;
    }

    // Viewport panning when animation is playing
    if (animation.value.isPlaying) {
      if (e.key === 'ArrowLeft') {
        panViewport('left');
        e.preventDefault();
      } else if (e.key === 'ArrowRight') {
        panViewport('right');
        e.preventDefault();
      } else if (e.key === 'ArrowUp') {
        panViewport('up');
        e.preventDefault();
      } else if (e.key === 'ArrowDown') {
        panViewport('down');
        e.preventDefault();
      }
    }
  };

  onMounted(() => {
    window.addEventListener('keydown', handleKeyDown);
  });

  onBeforeUnmount(() => {
    window.removeEventListener('keydown', handleKeyDown);
  });

  return {
    handleKeyDown,
  };
}
