import { ref, computed, type Ref } from 'vue';
import { type Sprite, type CellSize } from '@/application/types';

export function useSpritePosition(sprites: Ref<Sprite[]>, currentFrame: Ref<number>, cellSize: Ref<CellSize>, getSpriteOffset: (frameIndex: number) => { x: number; y: number }, onUpdateFrame: () => void, showNotification: (message: string, type?: 'success' | 'error') => void) {
  // State
  const isCanvasDragging = ref(false);
  const canvasDragStart = ref({ x: 0, y: 0 });

  // Computed
  const spriteOffset = computed({
    get: () => {
      // Get the offset for the current frame
      return getSpriteOffset(currentFrame.value);
    },
    set: val => {
      // Update the frame-specific offset directly
      const frameOffset = getSpriteOffset(currentFrame.value);
      frameOffset.x = val.x;
      frameOffset.y = val.y;

      // Re-render the frame
      onUpdateFrame();
    },
  });

  const hasSpriteOffset = computed(() => {
    return spriteOffset.value.x !== 0 || spriteOffset.value.y !== 0;
  });

  // Methods
  const resetSpritePosition = () => {
    // Get current sprite
    const sprite = sprites.value[currentFrame.value];
    if (!sprite) return;

    // Calculate center position
    const centerX = Math.max(0, Math.floor((cellSize.value.width - sprite.width) / 2));
    const centerY = Math.max(0, Math.floor((cellSize.value.height - sprite.height) / 2));

    // Update the sprite offset
    spriteOffset.value = { x: centerX, y: centerY };

    // Update the frame
    onUpdateFrame();

    // Show a notification
    showNotification('Sprite position reset to center');
  };

  const startCanvasDrag = (e: MouseEvent, isViewportDragging: Ref<boolean>, previewZoom: Ref<number>) => {
    if (sprites.value.length === 0) return;
    if (isViewportDragging.value) return;

    isCanvasDragging.value = true;

    // Store initial position
    canvasDragStart.value = {
      x: e.clientX,
      y: e.clientY,
    };

    // Add event listeners
    window.addEventListener('mousemove', e => handleCanvasDrag(e, previewZoom), { capture: true });
    window.addEventListener('mouseup', stopCanvasDrag, { capture: true });

    e.preventDefault();
    e.stopPropagation();
  };

  const handleCanvasDrag = (e: MouseEvent, previewZoom: Ref<number>) => {
    if (!isCanvasDragging.value) return;

    requestAnimationFrame(() => {
      // Get current sprite
      const sprite = sprites.value[currentFrame.value];
      if (!sprite) return;

      // Calculate delta from last position
      const deltaX = e.clientX - canvasDragStart.value.x;
      const deltaY = e.clientY - canvasDragStart.value.y;

      // Only move when delta exceeds the threshold for one pixel movement at current zoom
      const pixelThreshold = previewZoom.value; // One pixel at current zoom level

      // Calculate the maximum allowed offset
      const maxOffsetX = Math.max(0, cellSize.value.width - sprite.width);
      const maxOffsetY = Math.max(0, cellSize.value.height - sprite.height);

      // Move one pixel at a time when threshold is reached
      if (Math.abs(deltaX) >= pixelThreshold) {
        const pixelsToMove = Math.sign(deltaX);
        const newX = spriteOffset.value.x + pixelsToMove;
        spriteOffset.value.x = Math.max(0, Math.min(maxOffsetX, newX));

        // Reset the start X position for next pixel move
        canvasDragStart.value.x = e.clientX;
      }

      if (Math.abs(deltaY) >= pixelThreshold) {
        const pixelsToMove = Math.sign(deltaY);
        const newY = spriteOffset.value.y + pixelsToMove;
        spriteOffset.value.y = Math.max(0, Math.min(maxOffsetY, newY));

        // Reset the start Y position for next pixel move
        canvasDragStart.value.y = e.clientY;
      }

      // Update the frame
      onUpdateFrame();
    });
  };

  const stopCanvasDrag = (e: MouseEvent) => {
    if (!isCanvasDragging.value) return;

    isCanvasDragging.value = false;
    window.removeEventListener('mousemove', handleCanvasDrag as any, { capture: true });
    window.removeEventListener('mouseup', stopCanvasDrag, { capture: true });

    e.preventDefault();
    e.stopPropagation();
  };

  return {
    isCanvasDragging,
    canvasDragStart,
    spriteOffset,
    hasSpriteOffset,
    resetSpritePosition,
    startCanvasDrag,
    handleCanvasDrag,
    stopCanvasDrag,
  };
}
