import { ref, nextTick, type Ref } from 'vue';
import { type AnimationState } from '@/application/types';

export function useViewport(animation: Ref<AnimationState>, onUpdateFrame: () => void, showNotification: (message: string, type?: 'success' | 'error') => void) {
  // State
  const previewZoom = ref(1);
  const viewportOffset = ref({ x: 0, y: 0 });
  const isViewportDragging = ref(false);
  const viewportDragStart = ref({ x: 0, y: 0 });

  // Methods
  const zoomIn = () => {
    if (previewZoom.value < 5) {
      previewZoom.value = Math.min(5, previewZoom.value + 0.5);

      // Adjust container size after zoom change
      nextTick(() => {
        updateCanvasContainerSize();
      });
    }
  };

  const zoomOut = () => {
    if (previewZoom.value > 1) {
      previewZoom.value = Math.max(1, previewZoom.value - 0.5);

      // Adjust container size after zoom change
      nextTick(() => {
        updateCanvasContainerSize();
      });
    }
  };

  const resetZoom = () => {
    previewZoom.value = 1;
    viewportOffset.value = { x: 0, y: 0 };

    // Adjust container size after zoom change
    nextTick(() => {
      updateCanvasContainerSize();
    });
  };

  const updateCanvasContainerSize = () => {
    // This is a no-op in the composable, but can be implemented if needed
    // The actual container size is managed through reactive bindings in the template
  };

  const startViewportDrag = (e: MouseEvent, isCanvasDragging: Ref<boolean>) => {
    // Only enable viewport dragging when zoomed in
    if (previewZoom.value <= 1 || isCanvasDragging.value) return;

    isViewportDragging.value = true;
    viewportDragStart.value = {
      x: e.clientX,
      y: e.clientY,
    };

    // Add temporary event listeners
    window.addEventListener('mousemove', handleViewportDrag);
    window.addEventListener('mouseup', stopViewportDrag);

    // Prevent default to avoid text selection
    e.preventDefault();
  };

  const handleViewportDrag = (e: MouseEvent) => {
    if (!isViewportDragging.value) return;

    const deltaX = e.clientX - viewportDragStart.value.x;
    const deltaY = e.clientY - viewportDragStart.value.y;

    // Update viewport offset with the delta, scaled by zoom level
    viewportOffset.value = {
      x: viewportOffset.value.x + deltaX / previewZoom.value,
      y: viewportOffset.value.y + deltaY / previewZoom.value,
    };

    // Reset drag start position
    viewportDragStart.value = {
      x: e.clientX,
      y: e.clientY,
    };
  };

  const stopViewportDrag = () => {
    isViewportDragging.value = false;
    window.removeEventListener('mousemove', handleViewportDrag);
    window.removeEventListener('mouseup', stopViewportDrag);
  };

  const handleCanvasWheel = (e: WheelEvent) => {
    // Prevent the default scroll behavior
    e.preventDefault();

    if (e.ctrlKey) {
      // Ctrl + wheel = zoom in/out
      if (e.deltaY < 0) {
        zoomIn();
      } else {
        zoomOut();
      }
    } else {
      // Just wheel = pan when zoomed in
      if (previewZoom.value > 1) {
        // Adjust pan amount by zoom level
        const panFactor = 0.5 / previewZoom.value;

        if (e.shiftKey) {
          // Shift + wheel = horizontal pan
          viewportOffset.value.x -= e.deltaY * panFactor;
        } else {
          // Regular wheel = vertical pan
          viewportOffset.value.y -= e.deltaY * panFactor;
        }
      }
    }
  };

  const panViewport = (direction: 'left' | 'right' | 'up' | 'down', amount: number = 10) => {
    const panAmount = amount / previewZoom.value;

    switch (direction) {
      case 'left':
        viewportOffset.value.x += panAmount;
        break;
      case 'right':
        viewportOffset.value.x -= panAmount;
        break;
      case 'up':
        viewportOffset.value.y += panAmount;
        break;
      case 'down':
        viewportOffset.value.y -= panAmount;
        break;
    }
  };

  return {
    previewZoom,
    viewportOffset,
    isViewportDragging,
    zoomIn,
    zoomOut,
    resetZoom,
    updateCanvasContainerSize,
    startViewportDrag,
    handleViewportDrag,
    stopViewportDrag,
    handleCanvasWheel,
    panViewport,
  };
}
